#include <librealsense2/rs.hpp>
#include <pcl/io/pcd_io.h>
#include <pcl/point_types.h>
#include <pcl/filters/passthrough.h>
#include <pcl/visualization/pcl_visualizer.h>
#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
#include <iomanip>

//convenient typedefs
typedef pcl::PointXYZRGB PointT;
typedef pcl::PointCloud<PointT> PointCloud;
typedef pcl::PointXYZRGBNormal PointNormalT;
typedef pcl::PointCloud<PointNormalT> PointCloudWithNormals;
//timing
auto start = std::chrono::high_resolution_clock::now();
auto end = std::chrono::high_resolution_clock::now();

std::chrono::duration<double> duration;

void timer_start ()
{
  start = std::chrono::high_resolution_clock::now();
}

double timer ()
{
  end = std::chrono::high_resolution_clock::now();
  duration = end-start;
  return duration.count(); 
}

void rs2pc_to_pcl(const rs2::points& points, PointCloud::Ptr cloud)
{
  
}

std::string time_formating(double seconds)
{
	int milliseconds = int(seconds*1000);

	int ms = milliseconds % 1000;
	int s = ((milliseconds-ms)/1000)%60;
	int m = ((milliseconds-(s*1000)-ms)/60000); 

	std::stringstream ss;
	ss << m << ":"<< s << ":" << ms;

	return ss.str();
}

struct PCD{
	PointCloud::Ptr cloud;
	double time;
	PCD() : cloud (new PointCloud) {};
};

int main()
{

	// Start rs2 pipeline
	rs2::pipeline pipe;
	pipe.start();

	rs2::pointcloud pc;
	rs2::points points;

	rs2::frameset frames = pipe.wait_for_frames();

	

	int iter = 0;
	int init_iter = 30;

	std::cout << "Pointcloud Capture from Intel SR300" << std::endl;
	std::cout << "Initializing.." << std::endl;

	

	while(init_iter)
	{
		// Block program until frames arrive
		rs2::frameset frames = pipe.wait_for_frames();
		rs2::align align(RS2_STREAM_DEPTH);
		auto aligned_frames = align.process(frames);

		rs2::depth_frame aligned_depth_frame = aligned_frames.first(RS2_STREAM_DEPTH);
		rs2::video_frame color_frame = aligned_frames.get_color_frame();
		
		init_iter--;
	}


	boost::shared_ptr<pcl::visualization::PCLVisualizer> v (new pcl::visualization::PCLVisualizer ("3D Viewer"));


	
	std::cout << "Ready to capture pointclouds!" << std::endl;
	std::cout << "Press enter to continue" << std::endl;

	PointCloud::Ptr cloud (new PointCloud);

	while(std::cin.get() != '\n');

	timer_start();
	double time_now;

	double max_u = 0.0;
	double min_u = 999.0;

	double max_v = 0.0;
	double min_v = 999.0;

	while(1)
	{
		//cloud->empty();

		std::cout << "Frame " << iter << std::endl;
		// Block program until frames arrive
		// Try to get a frame of a depth image
		rs2::frameset frames = pipe.wait_for_frames();
		
		rs2::depth_frame aligned_depth_frame = frames.get_depth_frame();

		//rs2::depth_frame aligned_depth_frame = frames.get_depth_frame();
		//rs2::video_frame color_frame = frames.get_color_frame();

		time_now = timer();

		
		points = pc.calculate(aligned_depth_frame);

		//pc.map_to(color_frame);

		//const uint8_t  * color_image = ( const uint8_t  * )color_frame.get_data();
	
		auto sp = points.get_profile().as<rs2::video_stream_profile>();
		
		cloud->width = sp.width();
		cloud->height = sp.height();
		cloud->is_dense = false;
		cloud->points.resize(points.size());
		
		auto ptr = points.get_vertices();

		for(auto& p : cloud->points)
		{
		  p.x = ptr->x;
		  p.y = ptr->y;
		  p.z = ptr->z;

		  ptr++;
		}

		std::stringstream ss;

		ss << "Frame_" << iter << "_t" << time_formating(time_now) << ".pcd";

		

		//while(time_now - timer() < 0.1);
		pcl::PassThrough<PointT> pass;
		pass.setFilterFieldName("z");
		pass.setInputCloud(cloud);
		pass.setFilterLimits(0.0f, 1.5f);
		pass.filter(*cloud);

		pcl::io::savePCDFile (ss.str(),*cloud, true);
		iter++;
	}	 

	return 0;
}



